@@ -34,12 +34,13 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.5'
+__version__ = '0.6'
 
 
 import datetime
-import ldap
+import inspect
 import uuid
+import os
 
 class CredCache(object):
   """Credentials cache for XMLRPC services.
@@ -52,55 +53,66 @@ class CredCache(object):
   """
 
   # This will need a config option for the core objects, probably
-  def __init__(self, config_instance, inf_renew_time,
-               ldap_module=ldap):
+  def __init__(self, config_instance, inf_renew_time):
     """Constructs a new credential cache.
     Inputs:
       config_instance: instance of Config
       inf_renew_time: the that each credential is renewed (seconds)
-      ldap_module: module for ldap connection (used for unittest)
     """
     self.config_instance = config_instance
-    self.binddn = self.config_instance.config_file['credentials']['binddn']
     self.exp_time = self.config_instance.config_file['credentials']['exp_time']
-    self.ldap_module = ldap_module
+    self.authentication_method = self.config_instance.config_file[
+        'credentials']['authentication_method']
     self.inf_renew_time = inf_renew_time
 
     # garbage_collector contains cred strings in insertion order, to be walked
     # when it's time to remove potentially expired Credentials.
     self.garbage_collector = []
 
-  def Authenticate(self, user_name, password, server_address,
-                   cert_file='/etc/certs/cacert.pem'):
-    """Authenticates user against LDAP database
+  def Authenticate(self, user_name, password):
+    """Authenticates user against authentication method
 
     Inputs:
       user_name: string of user name
       password: string of password
-      server_address: string of LDAP server,
-                      ex: ldaps://ldap.university.edu:636
-      cert_file: string of locateion of alternate LDAP cert file
 
     Outputs:
       boolean of whether or not user is authenticated
     """
-    authenticated = False
-    binddn = self.binddn % (user_name)
-    self.ldap_module.set_option(ldap.OPT_X_TLS, 1)
-    self.ldap_module.set_option(ldap.OPT_X_TLS_CACERTFILE, cert_file)
-
-    ldap_server = self.ldap_module.initialize(server_address)
-    ldap_server.protocol_version = self.ldap_module.VERSION3
     try:
-      ldap_server.simple_bind_s(binddn, password)
-      authenticated = True
-    except self.ldap_module.LDAPError:
-      authenticated = False
-    ldap_server.unbind_s()
+      authenticate_module = __import__('roster_server.%s' % self.authentication_method)
+      authentication_module = getattr(authenticate_module, self.authentication_method)
+    except ImportError:
+      authentication_module = __import__(self.authentication_method)
+    authentication_module_instance = authentication_module.AuthenticationMethod()
+    authenticate_module_args = inspect.getargspec(
+        authentication_module_instance.Authenticate)[0]
+    if( authenticate_module_args[0] == 'self' ):
+      authenticate_module_args.pop(0)
+    for authenticate_module_arg in authenticate_module_args:
+       if( authenticate_module_arg == 'user_name' or
+           authenticate_module_arg == 'password' ):
+         continue
+       if( authenticate_module_arg not in self.config_instance.config_file[
+               self.authentication_method] ):
+         raise self.config_instance.ConfigError(
+             'Could not find "%s" value in "%s" in the "%s" section.' % (
+                 authenticate_module_arg, self.config_instance.config_file_path,
+                 self.authentication_method))
+    kwargs_dict = {}
+    for authenticate_module_arg in authenticate_module_args:
+      if( authenticate_module_arg == 'user_name' ):
+        kwargs_dict['user_name'] = user_name
+      elif( authenticate_module_arg == 'password' ):
+        kwargs_dict['password'] = password
+      else:
+        kwargs_dict[authenticate_module_arg] = (
+            self.config_instance.config_file[self.authentication_method][
+                authenticate_module_arg])
 
-    return authenticated
+    return authentication_module_instance.Authenticate(**kwargs_dict)
 
-  def CheckCredential(self, credential, core_instance):
+  def CheckCredential(self, credential, user_name, core_instance):
     """Checks users credential against database.
 
     Inputs:
@@ -135,6 +147,8 @@ class CredCache(object):
             return new_cred # Infinite key re issued
       return u'' # Infinite key is valid
     elif( db_timestamp > current_timestamp ):
+      if( credential_dict['user'] != user_name ):
+        return None
       core_instance._UpdateCredential(search_credential=credential,
                                       update_credential=credential)
       return u'' # Key is valid
@@ -142,13 +156,12 @@ class CredCache(object):
 
     return None # Key is expired
 
-  def GetCredentials(self, user_name, password, server_name, core_instance):
+  def GetCredentials(self, user_name, password, core_instance):
     """Return a valid credential string given a username and password.
 
     Inputs:
       user_name: strin of login for a user
       password: string of user's password
-      server_name: string of ldap server name
       core_instance: instance of Core
 
     Outputs:
@@ -160,7 +173,7 @@ class CredCache(object):
     """
     user_name = unicode(user_name)
     cred_string = ''
-    if( self.Authenticate(user_name, password, server_name) ):
+    if( self.Authenticate(user_name, password) ):
       current = core_instance._ListCredentials(user_name=user_name)
       if( current ):
         return current.keys()[0]
@@ -172,14 +185,4 @@ class CredCache(object):
           make_new_cred = False
 
     return cred_string
-
-  def ExpireCredentials(self):
-    """Expires all old credentials.  This changes internal state only, and
-    requires no inputs or outputs (and should raise no exceptions).
-    """
-
-    # Loop over garbage_collector array until a valid credential is found,
-    # deleting all expired credentials.
-    pass
-
 # vi: set ai aw sw=2:
