@@ -35,11 +35,10 @@ Allows client to connect and run arbitrary functions in core.py.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.5'
+__version__ = '0.6'
 
 
 import datetime
-import ldap
 import os
 import inspect
 import SocketServer
@@ -66,9 +65,9 @@ class ThreadedXMLRPCServer(SocketServer.ThreadingMixIn,SecureXMLRPCServer):
 
 class Server(object):
   """Daemon library used to serve commands to the client."""
-  def __init__(self, config_instance, keyfile, certfile,
+  def __init__(self, config_instance, keyfile=None, certfile=None,
                inf_renew_time=None, core_die_time=None,
-               clean_time=None, ldap_module=ldap, unittest_timestamp=None):
+               clean_time=None, unittest_timestamp=None):
     """Sets up config instance. Stores core instances.
 
     Inputs:
@@ -78,12 +77,18 @@ class Server(object):
        inf_renew_time: time to refresh infinite credentials (seconds)
        core_die_time: time for each core instance to die (seconds)
        clean_time: time to wait between core instance cleanings (seconds)
-       ldap_module: module used for ldap authentication (for testing)
     """
     self.config_instance = config_instance
     self.keyfile = keyfile
+    if( keyfile is None ):
+      self.keyfile = self.config_instance.config_file['server'][
+          'ssl_key_file']
     self.certfile = certfile
+    if( certfile is None ):
+      self.certfile = self.config_instance.config_file['server'][
+          'ssl_cert_file']
     self.inf_renew_time = inf_renew_time
+    self.core_store_cleanup_running = False
     if( inf_renew_time is None ):
       self.inf_renew_time = self.config_instance.config_file['server'][
           'inf_renew_time']
@@ -93,14 +98,13 @@ class Server(object):
           'core_die_time']
     self.get_credentials_wait_increment = self.config_instance.config_file[
         'server']['get_credentials_wait_increment']
-    self.ldap_server = self.config_instance.config_file['server']['ldap_server']
     self.server_killswitch = self.config_instance.config_file['server'][
         'server_killswitch']
     self.clean_time = clean_time
     if( clean_time is None ):
       self.clean_time = self.core_die_time
     self.cred_cache_instance = credentials.CredCache(self.config_instance,
-                                   inf_renew_time, ldap_module=ldap_module)
+                                   inf_renew_time)
     self.unittest_timestamp = unittest_timestamp
     self.core_store = [] # {'user': user, 'last_used': last_used, 'instance': }
     self.get_credentials_wait = {} # {'user1': 3, 'user2': 4}
@@ -119,34 +123,27 @@ class Server(object):
       for instance in delete_list:
         self.core_store.remove(instance)
 
-  def StringToUnicode(self, string):
-    """Returns a unicode string if string is not numeric
+  def StringToUnicode(self, object_to_convert):
+    """Converts objects recursively into strings.
 
     Inputs:
-      string: string to be used
-
+      object_to_convert: the object that needs to be converted to unicode
     Outputs:
-      string: either the same or unicode of above string
+      converted_object: object can vary type, but all strings will be unicode
     """
-    if( isinstance(string, str) ):
-      string = unicode(string)
-    elif( isinstance(string, list) ):
-      new_list = []
-      for item in string:
-        if( isinstance(item, dict) ):
-          new_dict = {}
-          for dict_item in item:
-            if( isinstance(item[dict_item], dict) ):
-              embedded_dict = {}
-              for dict_dict_item in item[dict_item]:
-                embedded_dict[unicode(dict_dict_item)] = unicode(item[
-                    dict_item][dict_dict_item])
-              new_dict[unicode(dict_item)] = embedded_dict
-            else:
-              new_dict[unicode(dict_item)] = unicode(item[dict_item])
-          new_list.append(new_dict)
-      string = new_list
-    return string
+    converted_object = object_to_convert
+    if( isinstance(object_to_convert, str) ):
+      converted_object = unicode(object_to_convert)
+    elif( isinstance(object_to_convert, dict) ):
+      new_dict = {}
+      for key, value in object_to_convert.iteritems():
+        new_dict[unicode(key)] = self.StringToUnicode(value)
+      converted_object = new_dict
+    elif( isinstance(object_to_convert, list) ):
+      for index, item in enumerate(object_to_convert):
+        object_to_convert[index] = self.StringToUnicode(item)
+      converted_object = object_to_convert
+    return converted_object
 
   def GetCoreInstance(self, user_name):
     """Finds core instance in core store, if one cannot be found
@@ -192,21 +189,18 @@ class Server(object):
     ## Instantiate the core instance
     core_instance = self.GetCoreInstance(user_name)
     core_helper_instance = roster_core.CoreHelpers(core_instance)
-    cred_status = self.cred_cache_instance.CheckCredential(credfile,
+    cred_status = self.cred_cache_instance.CheckCredential(credfile, user_name,
                                                            core_instance)
     if( cred_status is not None ):
       ## Fix non unicode strings containing no unicode characters
-      for arg in args:
-        args[args.index(arg)] = self.StringToUnicode(arg)
-        if( isinstance(arg, dict) ):
-          new_arg = {}
-          for i in arg:
-            new_arg.update(
-                {self.StringToUnicode(i): self.StringToUnicode(arg[i])})
-          args[args.index(arg)] = new_arg
+      args = self.StringToUnicode(args)
+      kwargs = self.StringToUnicode(kwargs)
 
-      for k in kwargs:
-        kwargs[k] = self.StringToUnicode(kwargs[k])
+      ## Fix unicoded kwargs keys
+      new_kwargs = {}
+      for key in kwargs:
+        new_kwargs[str(key)] = kwargs[key]
+      kwargs = new_kwargs
 
       if( function.startswith('_') ):
         raise FunctionError('Function does not exist.')
@@ -244,17 +238,14 @@ class Server(object):
     Inputs:
       user_name: string of user name
       password: string of password (plaintext)
-      server_name: ldap server url (string)
 
     Outputs:
       string: string of credential
               example: u'be4d4ecf-d670-44a0-b957-770e118e2755'
     """
-    server_name = self.ldap_server
     user_name = unicode(user_name)
     core_instance = self.GetCoreInstance(user_name)
     cred_string = self.cred_cache_instance.GetCredentials(user_name, password,
-                                                          server_name,
                                                           core_instance)
     if( cred_string == '' ):
       if( not self.get_credentials_wait.has_key(user_name) ):
@@ -280,7 +271,7 @@ class Server(object):
     credstring = unicode(credstring)
     core_instance = self.GetCoreInstance(user_name)
     valid = self.cred_cache_instance.CheckCredential(
-        credstring, core_instance)
+        credstring, user_name, core_instance)
     if( valid == '' ):
       return True
     return False
@@ -305,6 +296,9 @@ class Server(object):
     try:
       while 1:
         self.server.handle_request()
-        self.CleanupCoreStore()
+        if( not self.core_store_cleanup_running ):
+          self.core_store_cleanup_running = True
+          self.CleanupCoreStore()
+          self.core_store_cleanup_running = False
     except KeyboardInterrupt:
       print "Stopped by user."
